// Validtion tests for Ring Buffer
// Copyright (c) 2006 Atmel Corporation
// $Id: validate_ring_all.c,v 1.16 2007/05/01 17:55:22 eweddington Exp $

/**
*   @file
    Ring Buffer Validation Tests
    Version 1.1
    Updated 2006/08/31 by M. Vidales

*   \brief This file will run unit tests on the Ring Buffer
*
*/

#include <avr/io.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include "ring.h"

uint8_t data;


/** This will test that data can be added and removed properly from the ring
    buffer along with peeking ahead to verify proper data entry into the ring
    buffer. Upon the data being removed, there will be a comparison against the
    known values entered to test the overall process. If the process worked
    correctly, the test will return successful. */
bool test_ring_properly(void)
{
    uint8_t buffer[10];
    uint8_t buffer_temp[2];
    uint8_t peek_count = 0;
    bool init;
    bool init_success = false;
    ring_t ring;

    // Initialize the ring buffer to -> buffer[10]
    init = ring_init(&ring, buffer, 10);
    if(init)
    {
        // Add data to ring buffer
        ring_add(&ring, 13);
        ring_add(&ring, 10);

        // Peek ahead and validate data for next test process
        peek_count = ring_peek(&ring, buffer_temp, 2);
        bool test_peek = (bool)(buffer_temp[0] == 13 && buffer_temp[1] == 10);
        if(test_peek)
        {
            // Test first data entry
            data = ring_remove(&ring);
            if(data == 13)
            {
                // Test second data entry
                data = ring_remove(&ring);
                if(data == 10)
                {
                    // Test is validated by with proper operation
                    init_success = true;
                }
            }
        }
    }

    // Return test success
    return(init_success);
}

/** This test will verify the internal checking of NULL status within an input
    variable for ring_init and return false to the test_ring_null if performed
    correctly. The test will then complete successfully as long as the previous
    NULL test was false. */
bool test_ring_null(void)
{
    uint8_t buffer[10];
    bool init;
    bool init_success = false;
    ring_t ring;

    // Send a NULL value into ring_init in place of pointer ring
    init = ring_init(NULL, buffer, 10);
    if(init == false)
    {
        // Send a NULL value into ring_init in place of buffer
        init = ring_init(&ring, NULL, 10);
        if(init == false)
        {
            // Send a 0 value into ring_init in place of size
            init = ring_init(&ring, buffer, 0);
            if(init == false)
            {
                // Test is validated by 3 different NULL values
                init_success = true;
            }
        }
    }

    // Return test success
    return(init_success);
}

/** This test will verify that the NULL check is working correctly within the
    ring_clear function. If the check did it job correctly, the test will return
    successful. */
bool test_ring_clear(void)
{
    bool clear;
    bool clear_success = false;

    // Send a NULL value into ring_clear in place of pointer r
    clear = ring_clear(NULL);
    if(clear == false)
    {
        // Test is validated when ring_clear returns false
        clear_success = true;
    }

    // Return test success
    return(clear_success);
}

/** This test will verify the overall functionality of entering data into the
    ring buffer and having the pointers properly cycle around the end of the
    buffer size to continue entering data until the buffer is full. It will also
    verify that there is no missed data during the data entry process. Provided
    the overall functionality processed correctly, the test will return
    successful. */
bool test_ring_functionality(void)
{
    uint8_t buffer[10];
    uint8_t i = 0;
    uint8_t max;
    bool init;
    bool result;
    bool test_success;
    ring_t ring;

    // Initialize the ring buffer to -> buffer[10]
    init = ring_init(&ring, buffer, 10);

    // Assign new values for middle of buffer
    ring.head_index = 5;
    ring.tail_index = 5;
    ring.count = 0;

    // Add test data to the ring buffer
    do
    {
        result = ring_add(&ring, i);
        i++;
    }
    while(result);
    max = i-1;

    // Test removing data from the ring buffer. Should match what was put in
    // Here we will assume that the test always passes unless proven otherwise
    test_success = true;
    i = 0;
    while(ring.count && test_success)
    {
        data = ring_remove(&ring);
        if(data != i++)
        {
            // This will set the test success to false upon finding corrupt data
            test_success = false;
        }
    }
    // Verify that the count of the data is the same going in and coming out
    if(i != max)
    {
        test_success = false;
    }

    // Return test success
    return(test_success);
}

/** This test will verify that when NULL values are passed into the ring_peek
    function, the internal check will return false not allowing any invalid
    operations to occur. If the internal NULL test returns false, then the
    overall test will return successful. */
bool test_peek_null(void)
{
    uint8_t buffer_temp[2];
    int peek;
    bool peek_success = false;
    ring_t ring;

    // Send a value of NULL into ring_peek in place of pointer ring
    peek = ring_peek(NULL, buffer_temp, 2);
    if(peek == 0)
    {
        // Send a value of NULL into ring_peek in place of buffer_temp
        peek = ring_peek(&ring, NULL, 2);
        if(peek == 0)
        {
            // Test is validated by passing the 2 NULL tests
            peek_success = true;
        }
    }

	// Return test success
    return(peek_success);
}

/** This test will test the above_highmark and below_lowmark functions
    in the ring library. These functions can be called upon when a user
    may need flow control for the ring buffer. The test will allow data
    to be entered until the buffer reaches 75% capacity in which it will
    begin to remove data until the buffer reaches 25% capacity. If these
    functions perform as expected, the test will return successful. */
bool test_high_and_low_mark(void)
{
    uint8_t buffer[10];
    uint8_t i = 0;
    bool hi_mark;
    bool init;
    bool lo_mark;
    bool result;
    bool test_success = false;
    ring_t ring;

    // Inititalize the ring buffer for the hi_mark and lo_mark test
    init = ring_init(&ring, buffer, 10);
    if(init)
    {
        // Add test data into the buffer
        do
        {
            result = ring_add(&ring, i);
            // Check the pointer location and verify hi_mark
            hi_mark = ring_above_highmark(&ring);
            i++;
        }
        while(result && hi_mark == 0);

        // Remove test data from the buffer
        do
        {
            data = ring_remove(&ring);
            // Check the pointer location and verify lo_mark
            lo_mark = ring_below_lowmark(&ring);
        }
        while(lo_mark == 0);

        // Test that both hi_mark and lo_mark hold value until called again
        if(hi_mark && lo_mark)
        {
            // Test is validated by testing both levels of marks
            test_success = true;
        }
    }

    return(test_success);
}

/** This is the main process for testing all of the functions within the
    ring library. This will acitve LED's on the STK500 upon test success
    to validate the library functions. */
int main(void)
{
    DDRA = 0xFF;
    PORTA = 0xFF;

    bool test1;
    bool test2;
    bool test3;
    bool test4;
    bool test5;
    bool test6;

    test1 = test_ring_properly();       // Test data addtions and removals
    if(test1)
    {
        PORTA &= ~(1 << 0);             // If test passed true, turn on LED0
    }

    test2 = test_ring_null();           // Test passing of NULL values into ring_init
    if(test2)
    {
        PORTA &= ~(1 << 1);             // If test passed true, turn on LED1
    }

    test3 = test_ring_clear();          // Test passing of a NULL value into ring_clear
    if(test3)
    {
        PORTA &= ~(1 << 2);             // If test passed true, turn on LED2
    }

    test4 = test_ring_functionality();  // Test full funtionality of the ring buffer
    if(test4)
    {
        PORTA &= ~(1 << 3);             // If test passed true, turn on LED3
    }

    test5 = test_peek_null();           // Test passing of NULL values into ring_peek
    if(test5)
    {
    	PORTA &= ~(1 << 4);             // If test passed true, turn on LED4
    }

    test6 = test_high_and_low_mark();   // Test passing of NULL values into ring_peek
    if(test6)
    {
    	PORTA &= ~(1 << 5);             // If test passed true, turn on LED5
    }

    while(1)
    {
    }
}


