// Copyright (c) 2006 Atmel Corporation
// $Id: spi.h,v 1.7 2007/05/01 17:55:22 eweddington Exp $

#ifndef SPI_H
#define SPI_H

#include <avr/io.h>
#include <stdbool.h>
#include <stdint.h>

// Validate correct micro
#if defined(__AVR_ATmega640__) \
|| defined(__AVR_ATmega1280__) \
|| defined(__AVR_ATmega1281__) \
|| defined(__AVR_ATmega2560__) \
|| defined(__AVR_ATmega2561__) \
|| defined(__AVR_ATmega128__)

// Assert SPI port macros
#define SPI_PORT            PORTB
#define SPI_PORT_DIRECTION  DDRB
#define SPI_PIN             PINB
#define SPI_SLAVE           0
#define SPI_CLOCK           1
#define SPI_MOSI            2
#define SPI_MISO            3

#elif defined(__AVR_ATmega88__)

// Assert SPI port macros
#define SPI_PORT            PORTB
#define SPI_PORT_DIRECTION  DDRB
#define SPI_PIN             PINB
#define SPI_SLAVE           2
#define SPI_CLOCK           5
#define SPI_MOSI            3
#define SPI_MISO            4

#else
#error "MCU not supported yet!"
#endif


#define SPI_DUMMY_BYTE  ((uint8_t)0x00)
#define SPI_DUMMY_WORD  ((uint16_t)0x0000)

// Clock Divisions
typedef enum
{
    spi_clock_div_4 = 0,
    spi_clock_div_16 = 1,
    spi_clock_div_64 = 2,
    spi_clock_div_128 = 3
} spi_clock_t;



// API
// Initializes the SPI as a Master device.
bool spi_master_init(const spi_clock_t clock_div, const uint8_t data_mode, const bool data_order);

// Select the SPI data mode.
bool spi_data_mode(uint8_t data_mode);

// Set the SPI clock rate.
bool spi_clock_rate(uint8_t clock_div);

// This function controls the device selections within the SPI.
void spi_device(void (*select)(void), void (*deselect)(void));

/** This function selects the appropriate device for communication. */
void spi_device_select(void);

/** This function deselects the appropriate device from communication. */
void spi_device_deselect(void);

/** This function will control the transmit byte data within the SPI. */
uint8_t spi_transmit_byte(const uint8_t data);

/** This function will control the transmit word data within the SPI. */
uint16_t spi_transmit_word(const uint16_t data);

/** This function will control the receive byte data within the SPI. */
uint8_t spi_receive_byte(void);

/** This function will control the receive word data within the SPI. */
uint16_t spi_receive_word(void);

/** \ingroup libspi
    \brief Check if there was a SPI write collision.
*/
#define spi_write_collision()       (SPSR & (uint8_t)(1 << WCOL))

/** \ingroup libspi
    \brief Enable the SPI double speed.
*/
#define spi_speed_double_enable()   SPSR |= (uint8_t)((uint8_t)1 << SPI2X)

/** \ingroup libspi
    \brief Disable the SPI double speed.
*/
#define spi_speed_double_disable()  SPSR &= (uint8_t)~((uint8_t)1 << SPI2X)

/** \ingroup libspi
    \brief Read the SPI interrupt flag.
*/
#define spi_interrupt_flag()        (SPSR & (uint8_t)(1 << SPIF))

/** \ingroup libspi
    \brief Enable the SPI interrupt.
*/
#define spi_interrupt_enable()      SPCR |= (uint8_t)((uint8_t)1 << SPIE)

/** \ingroup libspi
    \brief Disable the SPI interrupt.
*/
#define spi_interrupt_disable()     SPCR &= (uint8_t)~((uint8_t)1 << SPIE)

/** \ingroup libspi
    \brief Enable the SPI module.
*/
#define spi_enable()                SPCR |= (uint8_t)((uint8_t)1 << SPE)

/** \ingroup libspi
    \brief Disable the SPI module.
*/
#define spi_disable()               SPCR &= (uint8_t)~((uint8_t)1 << SPE)

/** \ingroup libspi
    \brief Set the SPI data order to LSb.
*/
#define spi_data_order_lsb()        SPCR |= (uint8_t)((uint8_t)1 << DORD)

/** \ingroup libspi
    \brief Set the SPI data order to MSb.
*/
#define spi_data_order_msb()        SPCR &= (uint8_t)~((uint8_t)1 << DORD)

/** \ingroup libspi
    \brief Enable the SPI Master mode.
*/
#define spi_master_enable()         SPCR |= (uint8_t)((uint8_t)1 << MSTR)

/** \ingroup libspi
    \brief Disable the SPI Slave mode.
*/
#define spi_slave_enable()          SPCR &= (uint8_t)~((uint8_t)1 << MSTR)

/** \ingroup libspi
    \brief Set the SPI clock polarity leading edge to rising.
*/
#define spi_clock_polarity_leading_rising()     SPCR &= (uint8_t)~((uint8_t)1 << CPOL)

/** \ingroup libspi
    \brief Set the SPI clock polarity leading edge to falling.
*/
#define spi_clock_polarity_leading_falling()    SPCR |= (uint8_t)((uint8_t)1 << CPOL)

/** \ingroup libspi
    \brief Set the SPI clock phase leading edge to sample.
*/
#define spi_clock_phase_leading_sample()        SPCR &= (uint8_t)~((uint8_t)1 << CPHA)

/** \ingroup libspi
    \brief Set the SPI clock phase leading edge to setup.
*/
#define spi_clock_phase_leading_setup()         SPCR |= (uint8_t)((uint8_t)1 << CPHA)

/** \ingroup libspi
    \brief Select the slave device (attached to the ~SS line).
*/
#define spi_slave_select()          (SPI_PORT &= (uint8_t)~(1 << SPI_SLAVE))

/** \ingroup libspi
    \brief Deselect the slave device (attached to the ~SS line).
*/
#define spi_slave_deselect()        (SPI_PORT |= (uint8_t)(1 << SPI_SLAVE))

/** \ingroup libspi
    \brief Wait for the SPI interrupt flag to be set.
*/
#define spi_interrupt_flag_wait()   do{}while(!spi_interrupt_flag())

#endif
