// Copyright (c) 2006 Atmel Corporation
// $Id: spi_device.c,v 1.7 2007/05/01 17:55:22 eweddington Exp $

#define SPI_DEVICE

// Includes
#include <stdlib.h>
#include <avr/io.h>
#include "spi.h"


// Static Variables
static void (*device_select)(void);
static void (*device_deselect)(void);


// Implementation

/** \ingroup libspi
    \brief spi_device sets the selection and deselection functions for the
    slave device.
    \param select Pointer to a function that accepts no parameters and returns nothing. This points to the slave device selection function.
    \param deselect Pointer to a function that accepts no parameters and returns nothing. This points to the slave device deselection function.
    \note If the select parameter is set to NULL, then the spi_device_select() function will use the default ~SS line to select the slave device.
    \note If the deselect parameter is set to NULL, then the spi_device_deselect() function will use the default ~SS line to deselect the slave device.
*/
void spi_device(void (*select)(void), void (*deselect)(void))
{
    device_select = select;
    device_deselect = deselect;
    return;
}


/** \ingroup libspi
    \brief spi_device_select must be called before all SPI transmit or receive functions.
    \note The selection routine is set by calling the spi_device function.
*/
void spi_device_select(void)
{
    if (device_select == NULL)
    {
        // Use the default ~SS line to select slave.
        spi_slave_select();
    }
    else
    {
        // Call the user supplied function to select slave.
        (*device_select)();
    }
    return;
}


/** \ingroup libspi
    \brief spi_device_deselect must be called after all SPI transmit or receive functions.
    \note The deselection routine is set by calling the spi_device function.
*/
void spi_device_deselect(void)
{
    if (device_deselect == NULL)
    {
        // Use the default ~SS line to deselect slave.
        spi_slave_deselect();
    }
    else
    {
        // Call the user supplied function to deselect slave.
        (*device_deselect)();
    }
    return;
}
