// Copyright (c) 2006 Atmel Corporation
// $Id: spi_master_init.c,v 1.9 2007/05/01 17:55:22 eweddington Exp $

#define SPI_MASTER_INIT

/**
    \defgroup libspi
    
    libspi is a library for the on-board SPI peripheral for the AVR, for the GCC
    compiler.

    To use the API include this header file:
    \code
    #include "spi.h"
    \endcode

    Be sure to point to the location of the library file (libspi_<device>.a) 
    and the header file in your Project Settings, if you are using AVR Studio, 
    or in your Makefile, if you are using a simple Makefile-based project.
    
    Also be sure to use the correct library that matches your AVR device.
    
    Example of usage for a single slave device:
    \code
    #include <stdint.h>
    #include "spi.h"
    
    int main(void)
    {
        spi_master_init(spi_clock_div_4, 1, true);
        
        while(1)
        {
            foo();
            ...
        }
        return;
    }
    
    
    uint8_t foo(void)
    {
        spi_device_select();
        
        spi_transmit_byte(command);
        
        uint8_t data = spi_receive_byte();
        
        spi_device_deselect();
        
        return(data);
    }
    \endcode
    
    
    Example of usage for more than one slave device:
    \code
    #include <stdint.h>
    #include "spi.h"
    
    int main(void)
    {
        spi_master_init(spi_clock_div_4, 1, true);
        
        while(1)
        {
            foo();
            bar();
            ...
        }
        return;
    }
    
    
    uint8_t foo(void)
    {
        spi_device(foo_select, foo_deselect);
    
        spi_device_select();
        
        spi_transmit_byte(command);
        
        uint8_t data = spi_receive_byte();
        
        spi_device_deselect();
        
        return(data);
    }
    
    void foo_select(void)
    {
        spi_data_order_lsb();
        PORTA |= 0x80;
        return;
    }
    
    void foo_deselect(void)
    {
        PORTA &= (uint8_t)~0x80;
        return;
    }
    
    uint8_t bar(void)
    {
        spi_device(bar_select, bar_deselect);
    
        spi_device_select();
        
        spi_transmit_byte(command);
        
        uint8_t data = spi_receive_byte();
        
        spi_device_deselect();
        
        return(data);
    }
    
    void bar_select(void)
    {
        spi_data_order_msb();
        spi_slave_select();
        return;
    }
    
    void bar_deselect(void)
    {
        spi_slave_deselect();
        return;
    }
    
    \endcode
*/

#include <stdbool.h>
#include <stdint.h>
#include <stdlib.h>
#include "spi.h"

// Default values should be: polarity=false, phase=true, data order=false.

/** \ingroup libspi
    \brief spi_master_init will initialize a SPI master device. 
    \param clock_div Clock divisor to run the SPI module clock. Type spi_clock_t.
    \param data_mode The data mode for the SPI module. Acceptable values are [0-3]. See the AVR datasheet for details.
    \param data_order true = MSb first, false = LSb first.
    \return bool. true = success, false = failure.
*/
bool spi_master_init(const spi_clock_t clock_div, const uint8_t data_mode, const bool data_order)
{
    bool success = false;
    bool clock_rate_success;
    bool data_mode_success;

    spi_device(NULL, NULL);

    // Set output ports for SPI.
    SPI_PORT_DIRECTION |= (uint8_t)(((uint8_t)1 << SPI_SLAVE)
    | ((uint8_t)1 << SPI_CLOCK) | ((uint8_t)1 << SPI_MOSI));

    // Set input ports for SPI.
    SPI_PORT_DIRECTION &= (uint8_t)~((uint8_t)1 << SPI_MISO);

    // Enable master mode.
    spi_master_enable();

    // Set the SPI clock rate.
    clock_rate_success = spi_clock_rate(clock_div);
    if (clock_rate_success)
    {
        // Set the data mode (phase and polarity).
        data_mode_success = spi_data_mode(data_mode);
        if (data_mode_success)
        {
            // Set the order of data transmission.
            if (data_order)
            {
                spi_data_order_msb();
            }
            else
            {
                spi_data_order_lsb();
            }

            // Enable the SPI.
            spi_enable();
            success = true;
        }
    }
    return(success);
}


/** \ingroup libspi
    \brief spi_clock_rate will set the clock rate for the SPI.
    \param clock_div Clock divisor to run the SPI module clock. Type spi_clock_t.
    \return bool. true = success, false = failure.
*/
bool spi_clock_rate(spi_clock_t clock_div)
{
     bool success = false;

     // Test the input range
     if (clock_div < 4)
     {
        // Check bit 1 to determine SPR0 assertion
        if (clock_div & 0x01)
        {
            // Enable SPR0
            SPCR |= (uint8_t)(1 << SPR0);
        }
        else
        {
            // Disable SPR0
            SPCR &= (uint8_t)~(1 << SPR0);
        }

        // Check bit 2 to determine SPR1 assertion
        if (clock_div & 0x02)
        {
            // Enable SPR1
            SPCR |= (uint8_t)(1 << SPR1);
        }
        else
        {
            // Enable SPR0
            SPCR &= (uint8_t)~(1 << SPR1);
        }

        success = true;
     }
     return(success);
}


/** \ingroup libavr
    \brief spi_data_mode will set the SPI data mode, which includes the phase and polarity characteristics.
    \param data_mode The data mode for the SPI module. Acceptable values are [0-3]. See the AVR datasheet for details.
    \return bool. true = success, false = failure.
*/
bool spi_data_mode(uint8_t data_mode)
{
    bool success = false;

    // Test the input range
    if (data_mode < 4)
    {
        // Check bit 1 to determine CPHA assertion
        if (data_mode & 0x01)
        {
            // Enable CPHA
            SPCR |= (uint8_t)(1 << CPHA);
        }
        else
        {
            // Disable CPHA
            SPCR &= (uint8_t)~(1 << CPHA);
        }

        // Check bit 2 to determing CPOL assertion
        if (data_mode & 0x02)
        {
            // Enable CPOL
            SPCR |= (uint8_t)(1 << CPOL);
        }
        else
        {
            // Disable CPOL
            SPCR &= (uint8_t)~(1 << CPOL);
        }

        success = true;
    }
    return(success);
}
