// Validtion tests for SPI
// Copyright (c) 2006 Atmel Corporation
// $Id: validate_spi_all.c,v 1.7 2007/05/01 17:55:22 eweddington Exp $

/**
*   @file
    SPI Validation Tests
    Version 1.1
    Updated 2006/09/12 by M. Vidales

*   \brief This file will run unit tests on the SPI library
*
*/

#include <avr/io.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "spi.h"

/** This will test all internal checking of the spi_master_init function along
    with setting up the spi with appropriate values for proper operation in 
    communication between the microcontoller and the RF230 radio chip registers.
    If all test passed or failed correctly, the overall test will return
    successful. */
bool test_master_init(void)
{
    bool success = false;
    bool master_init_success;

    // Test the proper initialization values and watch for true passes
    master_init_success = spi_master_init(spi_clock_div_4, 0, false);
    if (master_init_success)
    {
        master_init_success = spi_master_init(spi_clock_div_16, 1, true);
        if (master_init_success)
        {
            master_init_success = spi_master_init(spi_clock_div_64, 2, false);
            if (master_init_success)
            {
                master_init_success = spi_master_init(spi_clock_div_128, 3, true);
                if (master_init_success)
                {
                    // Test the improper initialization values and watch for false passes
                    master_init_success = spi_master_init(4, 3, false);
                    if (master_init_success == false)
                    {
                        master_init_success = spi_master_init(spi_clock_div_128, 4, true);
                        if (master_init_success == false)
                        {
                            // Set actual initialization values for use in validating SPI library
                            master_init_success = spi_master_init(spi_clock_div_4, 0, true);
                            // Test is validated once passing all master_init tests
                            success = true;
                        }
                    }
                }
            }
        }
    }

    return(success);
}

/** This test will validate the proper operation of the spi_transmit_byte
    and spi_receive_byte functions. The test communicates to the registers
    within the RF230 radio chip to compare against actual known values. If
    the functions transmit and receive the expected values, the test will
    return as successful. */
bool test_read_spi_byte(void)
{
    bool success = false;
    uint8_t data[4];
    // Clear data buffer
    memset((void *)data, 0,4);
        
    // Set select line high to initialize SPI communication
    SPI_PORT |= (1 << PORTB0);
    // Set select line low to begin SPI communication
    SPI_PORT &= ~(1 << PORTB0);
    
    // Request a read of register 0x1C(PART_NUM)
    data[0] = spi_transmit_byte(0x9C);
    
    // Receive data requested and place into data buffer
    data[1] = spi_receive_byte();

    // Set select line high to end SPI communication
    SPI_PORT |= (1 << PORTB0);
    // Set select line low to begin new SPI communication
    SPI_PORT &= ~(1 << PORTB0);
    
    // Request a read of register 0x1D(VERSION_NUM)
    data[2] = spi_transmit_byte(0x9D);

    // Receive data requested and place into data buffer
    data[3] = spi_receive_byte();
        
    // Set select line high to end SPI communication
    SPI_PORT |= (1 << PORTB0);

    // Test received data against expected values
    if(data[1] == 2 && data[3] == 1)
    {
        // Test is validated once passing expected value comparison
        success = true;
    }

    return(success);
}

/** This test will validate the proper operation of the spi_transmit_word
    and spi_receive_word functions. The test communicates to the registers
    within the RF230 radio chip to compare against actual known values. If
    the functions transmit and receive the expected values, the test will
    return as successful. */ 
bool test_read_spi_word(void)
{
    bool success = false;
    // Initialize 16bit integers to use in data capture
    uint16_t data1 = 0;
    uint16_t data2 = 0;
    uint16_t data3 = 0;
    uint16_t data4 = 0;

    // Set select line high to initialize SPI communication 
    SPI_PORT |= (1 << PORTB0);  
    // Set select line low to begin SPI communication
    SPI_PORT &= ~(1 << PORTB0); 
    
    // Request a read of register 0x1C(PART_NUM) 
    data1 = spi_transmit_byte(0x009C);
    
    // Receive data requested and place into data buffer
    data2 = spi_receive_byte();

    // Set select line high to end SPI communication
    SPI_PORT |= (1 << PORTB0);  
    // Set select line low to begin new SPI communication
    SPI_PORT &= ~(1 << PORTB0); 
    
    // Request a read of register 0x1D(VERSION_NUM)
    data3 = spi_transmit_byte(0x009D);
    
    // Receive data requested and place into data buffer
    data4 = spi_receive_byte();

    // Set select line high to end SPI communication
    SPI_PORT |= (1 << PORTB0);  

    // Test received data against expected values
    if(data2 == 2 && data4 == 1)
    {
        // Test is validated once passing expected value comparison
        success = true;
    }

    return(success);
}

/** This is the main process for testing all of the functions within 
    the SPI library. This will activate LED's on the STK500 upon test
    success to validate the library functions. */
int main(void)
{
    DDRA = 0xFF;
    PORTA = 0xFF;

    bool test1;
    bool test2;
    bool test3;
    
    test1 = test_master_init();         // Test master_init internal checks
    if(test1)
    {
        PORTA &= ~(1 << 0);             // If test passed true, turn on LED0
    }

    test2 = test_read_spi_byte();       // Test spi_byte(transmit/receive) functions
    if(test2)
    {
        PORTA &= ~(1 << 1);             // If test passed true, turn on LED1
    }

    test3 = test_read_spi_word();       // Test spi_word(transmit/receive) functions
    if(test3)
    {
        PORTA &= ~(1 << 2);             // If test passed true, turn on LED2
    }

    while(1)
    {
    }
}
